# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/dataset.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)




DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/dataset.proto',
  package='irbench',
  serialized_pb='\n\x13proto/dataset.proto\x12\x07irbench\",\n\tSim2Query\x12\x0f\n\x07sim2qry\x18\x01 \x02(\x02\x12\x0e\n\x06imglst\x18\x02 \x03(\x05\"^\n\x05Query\x12\r\n\x05query\x18\x01 \x02(\x05\x12\x0e\n\x06poslst\x18\x02 \x03(\x05\x12\x0e\n\x06\x61mblst\x18\x03 \x03(\x05\x12&\n\nsim2qrylst\x18\x04 \x03(\x0b\x32\x12.irbench.Sim2Query\"M\n\x06Search\x12\x11\n\x05\x64\x62lst\x18\x01 \x03(\x05\x42\x02\x10\x01\x12\x1e\n\x06qrylst\x18\x02 \x03(\x0b\x32\x0e.irbench.Query\x12\x10\n\x08trainlst\x18\x07 \x03(\x05\"H\n\x07\x44\x61taset\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0f\n\x07\x63omment\x18\x02 \x02(\t\x12\x0e\n\x06images\x18\x03 \x03(\t\x12\x0e\n\x06labels\x18\x04 \x03(\x05')




_SIM2QUERY = _descriptor.Descriptor(
  name='Sim2Query',
  full_name='irbench.Sim2Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sim2qry', full_name='irbench.Sim2Query.sim2qry', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='imglst', full_name='irbench.Sim2Query.imglst', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=32,
  serialized_end=76,
)


_QUERY = _descriptor.Descriptor(
  name='Query',
  full_name='irbench.Query',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='query', full_name='irbench.Query.query', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='poslst', full_name='irbench.Query.poslst', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='amblst', full_name='irbench.Query.amblst', index=2,
      number=3, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sim2qrylst', full_name='irbench.Query.sim2qrylst', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=78,
  serialized_end=172,
)


_SEARCH = _descriptor.Descriptor(
  name='Search',
  full_name='irbench.Search',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dblst', full_name='irbench.Search.dblst', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
    _descriptor.FieldDescriptor(
      name='qrylst', full_name='irbench.Search.qrylst', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='trainlst', full_name='irbench.Search.trainlst', index=2,
      number=7, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=174,
  serialized_end=251,
)


_DATASET = _descriptor.Descriptor(
  name='Dataset',
  full_name='irbench.Dataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='irbench.Dataset.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='comment', full_name='irbench.Dataset.comment', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='images', full_name='irbench.Dataset.images', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='labels', full_name='irbench.Dataset.labels', index=3,
      number=4, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=253,
  serialized_end=325,
)

_QUERY.fields_by_name['sim2qrylst'].message_type = _SIM2QUERY
_SEARCH.fields_by_name['qrylst'].message_type = _QUERY
DESCRIPTOR.message_types_by_name['Sim2Query'] = _SIM2QUERY
DESCRIPTOR.message_types_by_name['Query'] = _QUERY
DESCRIPTOR.message_types_by_name['Search'] = _SEARCH
DESCRIPTOR.message_types_by_name['Dataset'] = _DATASET

class Sim2Query(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SIM2QUERY

  # @@protoc_insertion_point(class_scope:irbench.Sim2Query)

class Query(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _QUERY

  # @@protoc_insertion_point(class_scope:irbench.Query)

class Search(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SEARCH

  # @@protoc_insertion_point(class_scope:irbench.Search)

class Dataset(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DATASET

  # @@protoc_insertion_point(class_scope:irbench.Dataset)


_SEARCH.fields_by_name['dblst'].has_options = True
_SEARCH.fields_by_name['dblst']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')
# @@protoc_insertion_point(module_scope)
